<?php

namespace EventPortal\Domain\Agenda\Rules;

use EventPortal\Domain\Agenda\Models\Agenda;
use EventPortal\Domain\Agenda\Models\Pathway;
use EventPortal\Domain\Agenda\Models\Session;
use Illuminate\Contracts\Validation\Rule;
use Illuminate\Database\Eloquent\Builder;

class NoOverlap implements Rule
{
    /** @var \EventPortal\Domain\Agenda\Models\Pathway */
    private $pathway;

    /** @var EventPortal\Domain\Agenda\Models\Agenda */
    private $agenda;

    /** @var string */
    private $attribute;

    public function __construct(Pathway $pathway, Agenda $agenda)
    {
        $this->pathway = $pathway;
        $this->agenda = $agenda;
    }

    /**
     * @param string $attribute
     * @param mixed $value
     * @return bool
     */
    // phpcs:disable SlevomatCodingStandard.TypeHints.ReturnTypeHint
    // phpcs:disable SlevomatCodingStandard.TypeHints.ParameterTypeHint
    public function passes($attribute, $value)
    {
        // phpcs:enable
        $this->attribute = $attribute;

        return Session::query()
            ->where(function (Builder $query) use ($value) {
                return $query
                    ->forPathway($this->pathway)
                    ->forAgenda($this->agenda)
                    ->partiallyOverlaps($value);
            })
            ->exists() === false;
    }

    /**
     * @return array|string
     */
    public function message()
    {
        return "The {$this->getName()} cannot overlap another session.";
    }

    private function getName(): string
    {
        if ($this->attribute === 'starting_at') {
            return 'start time';
        }

        return 'end time';
    }
}
