<?php

namespace EventPortal\Domain\Util\Menu;

class Link
{
    /** @var null|string */
    public $name;

    /** @var null|string */
    public $icon;

    /** @var null|string */
    public $iconPosition;

    /** @var null|string */
    public $route;

    /** @var null|array */
    public $routeParams = [];

    /** @var bool */
    public $canSee = true;

    /**
     * @var string
     */
    public $href;

    public function __construct(?string $name = null)
    {
        $this->name = $name;
    }

    public static function make(?string $name = null): self
    {
        return new static($name);
    }

    public function name(string $name): self
    {
        $this->name = $name;

        return $this;
    }

    public function icon(string $icon, string $position = 'left'): self
    {
        $this->icon = $icon;
        $this->iconPosition = $position;

        return $this;
    }

    public function route(string $route, ?array $routeParams = []): self
    {
        $this->route = $route;
        $this->routeParams = $routeParams;

        return $this;
    }

    public function href(string $href): self
    {
        $this->href = $href;

        return $this;
    }

    /**
     * @param bool|callable $canSee
     * @return $this
     */
    public function canSee($canSee): self
    {
        if (is_callable($canSee)) {
            $this->canSee = $canSee();
            return $this;
        }

        $this->canSee = $canSee;

        return $this;
    }

    public function render(): array
    {
        return [
            'name' => $this->name,
            'route' => $this->route,
            'routeParams' => $this->routeParams,
            'href' => $this->href,
            'icon' => $this->icon,
            'iconPosition' => $this->iconPosition,
            'canSee' => $this->canSee,
        ];
    }
}
