<?php

namespace EventPortal\Http\Controllers\Admin;

use EventPortal\Domain\Agenda\Actions\CreateDefaultAgenda;
use EventPortal\Domain\Agenda\Actions\DeleteAgenda;
use EventPortal\Domain\Agenda\AgendasForBuilder;
use EventPortal\Domain\Agenda\Models\Agenda;
use EventPortal\Domain\Agenda\Requests\StoreAgendaRequest;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Util\AdminRender;
use EventPortal\Domain\Util\Breadcrumb;
use EventPortal\Domain\Util\QuickButtons\Facades\QuickButtons;
use EventPortal\Domain\Util\Sidebar\Facades\Sidebar;
use EventPortal\Http\Controllers\Controller;
use Illuminate\Http\RedirectResponse;
use Illuminate\Support\Arr;
use Inertia\Response;

class AgendaController extends Controller
{
    public function __construct()
    {
        $this->middleware('check-permission:agenda.show')->only('index');
        $this->middleware('check-permission:agenda.store')->only('store');
        $this->middleware('check-permission:agenda.destroy')->only('destroy');
    }

    public function index(Event $event): Response
    {
        $breadcrumbs = Breadcrumb::make()
            ->add('Events', route('admin.event.index'))
            ->add($event->name, route('admin.event.show', $event))
            ->add('Agenda')
            ->get();

        $sidebar = Sidebar::make('event.show')
            ->context(auth()->user())
            ->model($event)
            ->status(optional($event->approval)->getStatusTitleAttribute())
            ->render();

        $quickButtons = QuickButtons::make('agenda.index')->context($event)->get();

        return AdminRender::component('Admin/Agenda/Index')
            ->key('agenda.index')
            ->withData(AgendasForBuilder::get($event))
            ->attributes(['event' => $event])
            ->withQuickButton($quickButtons === [] ? null : head($quickButtons))
            ->withBreadcrumbs($breadcrumbs)
            ->withSideBar($sidebar)
            ->get();
    }

    public function store(
        StoreAgendaRequest $request,
        Event $event,
        CreateDefaultAgenda $createDefaultAgenda
    ): RedirectResponse {
        $createDefaultAgenda->execute($event, Arr::get($request->only('dates'), 'dates'));

        toast()->success(trans('event.agenda.created'));

        return redirect()->route('admin.agenda.index', ['event' => $event]);
    }

    public function destroy(Event $event, Agenda $agenda, DeleteAgenda $deleteAgenda): RedirectResponse
    {
        $deleteAgenda->execute($agenda);

        toast()->success(trans('event.agenda.deleted'));

        return redirect()->route('admin.agenda.index', ['event' => $event]);
    }
}
