<?php

namespace EventPortal\Tests\Unit\Component;

use EventPortal\Domain\Admin\Models\Admin;
use EventPortal\Domain\Component\Button;
use EventPortal\Domain\Component\ButtonDropdown;
use EventPortal\Domain\Permission\Models\Permission;
use EventPortal\Enums\InertiaMethod;
use EventPortal\Tests\Support\Traits\SeedRolesAndPermissions;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;
use Spatie\Snapshots\MatchesSnapshots;

class ButtonDropdownTest extends TestCase
{
    use RefreshDatabase, MatchesSnapshots, SeedRolesAndPermissions;

    /** @test **/
    public function can_make_button_dropdown()
    {
        $dropdown = ButtonDropdown::make('Manage delegates')
            ->actions([
                Button::make('Add')->method(InertiaMethod::POST()),
                Button::make('Delete')->method(InertiaMethod::POST()),
            ]);

        $this->assertEquals('ButtonDropdown', $dropdown->component);
        $this->assertCount(2, $dropdown->actions);
    }

    /** @test */
    public function can_add_actions()
    {
        $buttonDropdown = ButtonDropdown::make('Manage event')
            ->actions([
                Button::make('View'),
                Button::make('Delete'),
            ]);

        $this->assertMatchesJsonSnapshot(json_encode($buttonDropdown->actions));
    }

    /** @test */
    public function adds_when_user_has_permissions()
    {
        $admin = factory(Admin::class)->create();
        $this->actingAsAdmin($admin);

        factory(Permission::class)->create([
            'name' => 'can add button',
            'guard_name' => 'web',
        ]);

        $admin->givePermissionTo('can add button');

        $buttonDropdown = ButtonDropdown::make('Button')
            ->when(has_permission('can add button'), function (ButtonDropdown $button): void {
                $button->actions([
                    Button::make('View'),
                ]);
            });

        $this->assertMatchesJsonSnapshot(json_encode($buttonDropdown->actions));
    }

    /** @test */
    public function does_not_add_when_user_does_not_have_permission()
    {
        $admin = factory(Admin::class)->create();
        $this->actingAsSuperAdmin($admin);

        $button = ButtonDropdown::make('Button')
            ->when(has_permission('can add button'), function (ButtonDropdown $button) {
                $button->actions([
                    Button::make('View'),
                ]);
            });

        $this->assertEmpty($button->actions);
    }
}
