<?php

namespace EventPortal\Tests\Unit\Component\Filter;

use EventPortal\Domain\Component\Filter\StatusFilter;
use EventPortal\Tests\Support\TestModel;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class StatusFilterTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function type_is_correct(): void
    {
        $filter = StatusFilter::make('Status');

        $this->assertSame('MultiSelect', $filter->type);
    }

    /** @test **/
    public function can_filter_by_single_choice(): void
    {
        $eventA = factory(TestModel::class)->create(['status' => 'status_a']);
        $eventB = factory(TestModel::class)->create(['status' => 'status_b']);

        $query = TestModel::query();

        $filter = StatusFilter::make('Status');

        $query = $filter->lookup->__invoke($query, 'status_b', 'status');

        $actual = $query->get();

        $this->assertNotContainsModel($eventA, $actual);
        $this->assertContainsModel($eventB, $actual);
    }

    /** @test **/
    public function can_filter_by_multi_choice(): void
    {
        $eventA = factory(TestModel::class)->create(['status' => 'status_a']);
        $eventB = factory(TestModel::class)->create(['status' => 'status_b']);
        $eventC = factory(TestModel::class)->create(['status' => 'status_c']);

        $query = TestModel::query();

        $filter = StatusFilter::make('Status');

        $query = $filter->lookup->__invoke($query, ['status_b', 'status_c'], 'status');

        $actual = $query->get();

        $this->assertNotContainsModel($eventA, $actual);
        $this->assertContainsModel($eventB, $actual);
        $this->assertContainsModel($eventC, $actual);
    }
}
