<?php

namespace EventPortal\Tests\Unit\EventDelegate\Actions;

use EventPortal\Domain\Agenda\Models\Pathway;
use EventPortal\Domain\Delegate\Models\Delegate;
use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\EventDelegate\Actions\AssociateToEvent;
use EventPortal\Domain\EventDelegate\Actions\DisassociateFromEvent;
use EventPortal\Domain\EventDelegate\Events\EventDelegatesDisassociatedFromEvent;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class DisassociateFromEventTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function disassociate_delegates_from_event(): void
    {
        $event = factory(Event::class)->create();
        Pathway::generate($event, Pathway::DEFAULT, 'Default pathway');
        [$delegateA, $delegateB, $delegateC] = factory(Delegate::class, 3)->create();

        (new AssociateToEvent)->execute($event, [
            $delegateA->id,
            $delegateB->id,
            $delegateC->id,
        ]);

        (new DisassociateFromEvent)->execute($event, [
            $delegateA->id,
            $delegateC->id,
        ]);

        $this->assertEventMissingDelegates($event, [$delegateA, $delegateC]);
        $this->assertEventHasDelegate($event, $delegateB);
    }

    /** @test **/
    public function event_is_dispatched(): void
    {
        \Illuminate\Support\Facades\Event::fake();

        $event = factory(Event::class)->create();

        Pathway::generate($event, Pathway::DEFAULT, 'The default pathway');

        $delegate = factory(Delegate::class)->create();

        (new DisassociateFromEvent)->execute($event, [$delegate->id]);

        \Illuminate\Support\Facades\Event::assertDispatched(EventDelegatesDisassociatedFromEvent::class);
    }
}
