<?php

namespace EventPortal\Tests\Unit\EventVenue\Actions;

use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventVenue;
use EventPortal\Domain\EventVenue\Actions\AssociateToEvent;
use EventPortal\Domain\EventVenue\Events\EventVenuesAssociatedToEvent;
use EventPortal\Domain\Venue\Models\Venue;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class AssociateToEventTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function associate_to_event(): void
    {
        $event = factory(Event::class)->create();
        [$venueA, $venueB, $venueC] = factory(Venue::class, 3)->create();

        (new AssociateToEvent)->execute($event, [
            $venueA->id,
            $venueC->id,
        ]);

        $this->assertEventHasVenues($event, [$venueA, $venueC]);
        $this->assertEventMissingVenue($event, $venueB);
    }

    /** @test **/
    public function does_not_associate_more_than_once(): void
    {
        $event = factory(Event::class)->create();
        $venue = factory(Venue::class)->create();

        (new AssociateToEvent)->execute($event, [$venue->id]);
        (new AssociateToEvent)->execute($event, [$venue->id]);

        $this->assertCount(1, EventVenue::all());
    }

    /** @test **/
    public function event_is_dispatched(): void
    {
        \Illuminate\Support\Facades\Event::fake();

        $event = factory(Event::class)->create();

        $venue = factory(Venue::class)->create();

        (new AssociateToEvent())->execute($event, [$venue->id]);

        \Illuminate\Support\Facades\Event::assertDispatched(EventVenuesAssociatedToEvent::class);
    }
}
