<?php

namespace EventPortal\Tests\Unit\EventVenue\Actions;

use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\EventVenue\Actions\AssociateToEvent;
use EventPortal\Domain\EventVenue\Actions\DisassociateFromEvent;
use EventPortal\Domain\EventVenue\Events\EventVenuesDisassociatedFromEvent;
use EventPortal\Domain\Venue\Models\Venue;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class DisassociateFromEventTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function disassociate_from_event(): void
    {
        $event = factory(Event::class)->create();
        [$venueA, $venueB, $venueC] = factory(Venue::class, 3)->create();

        (new AssociateToEvent)->execute($event, [
            $venueA->id,
            $venueB->id,
            $venueC->id,
        ]);

        (new DisassociateFromEvent)->execute($event, [
            $venueA->id,
            $venueC->id,
        ]);

        $this->assertEventMissingVenues($event, [$venueA, $venueC]);
        $this->assertEventHasVenue($event, $venueB);
    }

    /** @test **/
    public function event_is_dispatched(): void
    {
        \Illuminate\Support\Facades\Event::fake();

        $event = factory(Event::class)->create();

        $venue = factory(Venue::class)->create();

        (new DisassociateFromEvent())->execute($event, [$venue->id]);

        \Illuminate\Support\Facades\Event::assertDispatched(EventVenuesDisassociatedFromEvent::class);
    }
}
