<?php

namespace EventPortal\Tests\Unit\Speaker\Actions;

use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\Event\Models\EventSpeaker;
use EventPortal\Domain\EventSpeaker\Actions\AssociateToEvent;
use EventPortal\Domain\EventSpeaker\Events\EventSpeakersAssociatedToEvent;
use EventPortal\Domain\EventVenue\Events\EventVenuesAssociatedToEvent;
use EventPortal\Domain\Speaker\Models\Speaker;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class AssociateToEventTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function associate_to_event(): void
    {
        $event = factory(Event::class)->create();

        [$speakerA, $speakerB, $speakerC] = factory(Speaker::class, 3)->create();

        (new AssociateToEvent)->execute($event, [
            $speakerA->id,
            $speakerC->id,
        ]);

        $this->assertEventHasSpeakers($event, [$speakerA, $speakerC]);
        $this->assertEventMissingSpeaker($event, $speakerB);
    }

    /** @test **/
    public function does_not_associate_more_than_once(): void
    {
        $event = factory(Event::class)->create();

        $speaker = factory(Speaker::class)->create();

        (new AssociateToEvent)->execute($event, [$speaker->id]);
        (new AssociateToEvent)->execute($event, [$speaker->id]);

        $this->assertCount(1, EventSpeaker::all());
    }

    /** @test **/
    public function event_is_dispatched(): void
    {
        \Illuminate\Support\Facades\Event::fake();

        $event = factory(Event::class)->create();

        $speaker = factory(Speaker::class)->create();

        (new AssociateToEvent())->execute($event, [$speaker->id]);

        \Illuminate\Support\Facades\Event::assertDispatched(EventSpeakersAssociatedToEvent::class);
    }
}
