<?php

namespace EventPortal\Tests\Unit\Speaker\Actions;

use EventPortal\Domain\Event\Models\Event;
use EventPortal\Domain\EventSpeaker\Actions\AssociateToEvent;
use EventPortal\Domain\EventSpeaker\Actions\DisassociateFromEvent;
use EventPortal\Domain\EventSpeaker\Events\EventSpeakersDisassociatedFromEvent;
use EventPortal\Domain\Speaker\Models\Speaker;
use EventPortal\Tests\TestCase;
use Illuminate\Foundation\Testing\RefreshDatabase;

class DisassociateFromEventTest extends TestCase
{
    use RefreshDatabase;

    /** @test **/
    public function disassociate_from_event(): void
    {
        $event = factory(Event::class)->create();
        [$speakerA, $speakerB, $speakerC] = factory(Speaker::class, 3)->create();

        (new AssociateToEvent)->execute($event, [
            $speakerA->id,
            $speakerB->id,
            $speakerC->id,
        ]);

        (new DisassociateFromEvent)->execute($event, [
            $speakerA->id,
            $speakerC->id,
        ]);

        $this->assertEventMissingSpeakers($event, [$speakerA, $speakerC]);
        $this->assertEventHasSpeaker($event, $speakerB);
    }

    /** @test **/
    public function event_is_dispatched(): void
    {
        \Illuminate\Support\Facades\Event::fake();

        $event = factory(Event::class)->create();

        $venue = factory(Speaker::class)->create();

        (new DisassociateFromEvent())->execute($event, [$venue->id]);

        \Illuminate\Support\Facades\Event::assertDispatched(EventSpeakersDisassociatedFromEvent::class);
    }
}
